#include "project.h"
#include "stdio.h"
#include "tty_utils.h"
#include "mem_utils.h"

static int mem_keep_on = 1 ;

f_list_type mem_func_list[] = {
    { "help",   do_mem_help,   "show help message" },
    { "read",   do_mem_read,   "read memory" },
    { "write",  do_mem_write,  "write memory" },
    { "dump",   do_mem_dump,   "dump memory" },
    { "return", do_mem_return, "return to the previous mode" },
    { 0,        0,             0} /* list terminator */
} ;
void do_mem(void)
{
    func_ptr func ;
    mem_keep_on = 1 ;
    
    do_mem_help() ;
    
    prompt("mem> ") ;
    
    while(mem_keep_on) {
        if (get_line()) {
            func = get_func(mem_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_mem_help() ;
            }
            if (mem_keep_on) {
                prompt("mem> ") ;
            }
        }
    }
}


void do_mem_help(void)
{
    show_help(mem_func_list) ;
}

void do_mem_read(void)
{
    uint32_t address ;
    uint8_t value ;
    sscanf(str, "%*s %x", &address) ;
    value = *((uint8_t *)address) ;
    snprintf(str, STR_BUF_LEN, "0x%08X : 0x%02X\n\r", address, value) ;
    print(str) ;
}

void do_mem_write(void)
{
    uint32_t address ;
    uint32_t value ;
    sscanf(str, "%*s %x %x", &address, &value) ;
    *((uint8_t *)(address)) = (value & 0xFF) ;
}

void do_mem_dump(void)
{
    int row, col ;
    uint32_t address ;
    uint8_t value ;
    sscanf(str, "%*s %x", &address) ;
    address &= 0xFFFFFF00 ; /* 256byte boundary */
    for (row = 0 ; row < 0x10 ; row++ ) {
        snprintf(str, STR_BUF_LEN, "0x%08X: ", address + 0x10 * row) ;
        print(str) ;
        for (col = 0 ; col < 0x10 ; col++ ) {
            value = *((uint8_t *)(address + (0x10 * row) + col)) ;
            snprintf(str, STR_BUF_LEN, "%02X ", value) ;
            print(str) ;
        }
        print(":") ;
        for (col = 0 ; col < 0x10 ; col++) {
              value = *((uint8_t *)(address + (0x10 * row) + col)) ;
            if (isprint(value)) {
                snprintf(str, STR_BUF_LEN, "%c", value) ;
                print(str) ;
            } else {
                print(".") ;
            }
        }
        print(":\n\r") ;
    }
}

void do_mem_return(void) 
{
    mem_keep_on = 0 ;
}