#include "project.h"
#include "stdio.h"
#include "tty_utils.h"
#include "pwm_utils.h"

static int      pwm_keep_on = 1 ;
static int      pwm_running = 1 ;
static uint16_t pwm_period ;
static uint16_t pwm_compare ;

f_list_type pwm_func_list[] = {
    { "help",    do_pwm_help,    "show help message" },
    { "config",  do_pwm_config,  "set pwm period and compare" },
    { "period",  do_pwm_period,  "set pwm period" },
    { "compare", do_pwm_compare, "set pwm compare" },
    { "start",   do_pwm_start,   "start pwm"},
    { "stop",    do_pwm_stop,    "stop pwm"},
    { "status",  do_pwm_status,  "report pwm status" },
    { "return",  do_pwm_return,  "return to the previous mode" },
    { 0,         0,             0} /* list terminator */
} ;

void do_pwm(void)
{
    func_ptr func ;
    pwm_keep_on = 1 ;
    
    PWM_Start() ;
    
    do_pwm_help() ;
    
    prompt("pwm> ") ;
    
    while(pwm_keep_on) {
        if (get_line()) {
            func = get_func(pwm_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_pwm_help() ;
            }
            if (pwm_keep_on) {
                prompt("pwm> ") ;
            }
        }
    }
}


void do_pwm_help(void) 
{
    show_help(pwm_func_list) ;    
}

void do_pwm_return(void) 
{
    pwm_keep_on = 0 ;
}

void config_pwm(uint16_t period, uint16_t compare)
{
    pwm_period = period ;
    pwm_compare = compare ;
    PWM_Stop() ;
    PWM_WritePeriod(pwm_period) ;
    PWM_WriteCompare(pwm_compare) ;
    PWM_WriteCounter(0) ;
    if (pwm_running) {
        PWM_Enable() ;
    }    
}

void do_pwm_config(void)
{
    uint16_t period, compare ;
    sscanf(str, "%*s %hd %hd", &period, &compare) ;
    config_pwm(period, compare) ;
}

void do_pwm_period(void) 
{
    uint16_t period ;
    sscanf(str, "%*s %hd", &period) ;
    config_pwm(period, pwm_compare) ;    
}

void do_pwm_compare(void) 
{
    uint16_t compare ;
    sscanf(str, "%*s %hd", &compare) ;
    config_pwm(pwm_period, compare) ;     
}

void do_pwm_start(void) 
{
    pwm_running = 1 ;
    PWM_Enable() ;
}

void do_pwm_stop(void) 
{
    pwm_running = 0 ;
    PWM_Stop() ;
}

void do_pwm_status(void)
{
    pwm_period = PWM_ReadPeriod() ;
    pwm_compare = PWM_ReadCompare() ;
    snprintf(str, STR_BUF_LEN, "PWM: period: %hu compare: %hu ",
        pwm_period, pwm_compare) ;
    print(str) ;
    print("status: ") ;
    if (pwm_running) {
        print("Running") ;
    } else {
        print("Not Running") ;
    }
    print("\n\r") ;
}